@@ -70,8 +70,14 @@ void Editor::setupUi() {
                             { "Insert C<m>h</m>aracter...", "", "InsertCharacter", {}},
                             {},
                             { "<m>G</m>oto Line", "Ctrl-G", "Gotoline", {}},
+                            { "Set Marker", "Ctrl-M", "addLineMarker", {}},
+                            // { "Marker", "", "", {
+                                //{ "Set Marker", "Ctrl-M", "addLineMarker", {}},
+                                //{ "Goto Next Marker", "Ctrl-,", "nextLineMarker", {}},
+                                //{ "Goto Previous Marker", "Ctrl-.", "previousLineMarker", {}},
+                            // }},
                             {},
-                            { "Sort Selcted Lines", "Alt-Shift-S", "SortSelectedLines", {}}
+                            { "Sort Selected Lines", "Alt-Shift-S", "SortSelectedLines", {}}
                         }
                       },
                       { "<m>O</m>ptions", "", {}, {
@@ -159,6 +165,26 @@ void Editor::setupUi() {
     _cmdGotoLine = new Tui::ZCommandNotifier("Gotoline", this);
     QObject::connect(_cmdGotoLine, &Tui::ZCommandNotifier::activated, this, gotoLine);
 
+    //Marker
+    _cmdLineMarker = new Tui::ZCommandNotifier("addLineMarker", this);
+    QObject::connect(_cmdLineMarker, &Tui::ZCommandNotifier::activated, this, [this] {
+        if (_file) {
+            _file->toggleLineMarker();
+        }
+    });
+    _cmdNextLineMarker = new Tui::ZCommandNotifier("nextLineMarker", this);
+    QObject::connect(_cmdNextLineMarker, &Tui::ZCommandNotifier::activated, this, [this] {
+        if (_file) {
+            _file->gotoNextLineMarker();
+        }
+    });
+    _cmdPreviousLineMarker = new Tui::ZCommandNotifier("previousLineMarker", this);
+    QObject::connect(_cmdPreviousLineMarker, &Tui::ZCommandNotifier::activated, this, [this] {
+        if (_file) {
+            _file->gotoPreviousLineMarker();
+        }
+    });
+
     //Sort Seleced Lines
     _cmdSortSelectedLines = new Tui::ZCommandNotifier("SortSelectedLines", this);
     QObject::connect(_cmdSortSelectedLines, &Tui::ZCommandNotifier::activated, this, [this] {
@@ -267,12 +293,14 @@ void Editor::setupUi() {
     QObject::connect(new Tui::ZShortcut(Tui::ZKeySequence::forMnemonic("x"), this, Qt::ApplicationShortcut), &Tui::ZShortcut::activated,
                      this, &Editor::showCommandLine);
 
-    QObject::connect(new Tui::ZCommandNotifier("NextWindow", this), &Tui::ZCommandNotifier::activated, this,
+    _cmdNextWindow = new Tui::ZCommandNotifier("NextWindow", this);
+    QObject::connect(_cmdNextWindow, &Tui::ZCommandNotifier::activated, this,
         [this] {
             activateNextWindow();
         }
     );
-    QObject::connect(new Tui::ZCommandNotifier("PreviousWindow", this), &Tui::ZCommandNotifier::activated, this,
+    _cmdPreviousWindow = new Tui::ZCommandNotifier("PreviousWindow", this);
+    QObject::connect(_cmdPreviousWindow, &Tui::ZCommandNotifier::activated, this,
         [this] {
             activatePreviousWindow();
         }
@@ -281,30 +309,35 @@ void Editor::setupUi() {
     _cmdTileVert = new Tui::ZCommandNotifier("TileVert", this);
     QObject::connect(_cmdTileVert, &Tui::ZCommandNotifier::activated, this, [this] {
             _mdiLayout->setMode(MdiLayout::LayoutMode::TileV);
+            updateLayoutMode();
         }
     );
 
     QObject::connect(new Tui::ZShortcut(Tui::ZKeySequence::forShortcutSequence("e", Qt::ControlModifier, "v", {}), this, Qt::ApplicationShortcut), &Tui::ZShortcut::activated,
         this, [this] {
             _mdiLayout->setMode(MdiLayout::LayoutMode::TileV);
+            updateLayoutMode();
         }
     );
 
     _cmdTileHorz = new Tui::ZCommandNotifier("TileHorz", this);
     QObject::connect(_cmdTileHorz, &Tui::ZCommandNotifier::activated, this, [this] {
             _mdiLayout->setMode(MdiLayout::LayoutMode::TileH);
+            updateLayoutMode();
         }
     );
 
     QObject::connect(new Tui::ZShortcut(Tui::ZKeySequence::forShortcutSequence("e", Qt::ControlModifier, "h", {}), this, Qt::ApplicationShortcut), &Tui::ZShortcut::activated,
         this, [this] {
             _mdiLayout->setMode(MdiLayout::LayoutMode::TileH);
+            updateLayoutMode();
         }
     );
 
     _cmdTileFull = new Tui::ZCommandNotifier("TileFull", this);
     QObject::connect(_cmdTileFull, &Tui::ZCommandNotifier::activated, this, [this] {
             _mdiLayout->setMode(MdiLayout::LayoutMode::Base);
+            updateLayoutMode();
         }
     );
 
@@ -414,8 +447,10 @@ FileWindow *Editor::createFileWindow() {
     FileWindow *win = new FileWindow(this);
     File *file = win->getFileWidget();
 
+    _mux.connect(win, file, &File::cursorPositionChanged, this, &Editor::updateTerminalHeight);
     _mux.connect(win, file, &File::cursorPositionChanged, _statusBar, &StatusBar::cursorPosition, 0, 0, 0, 0);
     _mux.connect(win, file, &File::scrollPositionChanged, _statusBar, &StatusBar::scrollPosition, 0, 0);
+    _mux.connect(win, file, &File::selectCharLines, _statusBar, &StatusBar::setSelectCharLines, 0, 0);
     _mux.connect(win, file, &File::modifiedChanged, _statusBar, &StatusBar::setModified, false);
     _mux.connect(win, win, &FileWindow::readFromStandadInput, _statusBar, &StatusBar::readFromStandardInput, false);
     //_mux.connect(win, win, &FileWindow::followStandadInput, _statusBar, &StatusBar::followStandardInput, false);
@@ -532,7 +567,6 @@ QVector<Tui::ZMenuItem> Editor::createWindowMenu() {
                                          {}});
             }
         }
-
     }
 
     return subitems;
@@ -669,17 +703,24 @@ void Editor::ensureWindowCommands(int count) {
 void Editor::enableFileCommands(bool enable) {
     _cmdInsertCharacter->setEnabled(enable);
     _cmdGotoLine->setEnabled(enable);
+    _cmdLineMarker->setEnabled(enable);
+    _cmdNextLineMarker->setEnabled(enable);
+    _cmdPreviousLineMarker->setEnabled(enable);
     _cmdSortSelectedLines->setEnabled(enable);
     _cmdTab->setEnabled(enable);
     _cmdLineNumbers->setEnabled(enable);
-    //_cmdFormatting->setEnabled(enable);
+    // _cmdFormatting works without active document and should be kept enabled
     _cmdBrackets->setEnabled(enable);
 #ifdef SYNTAX_HIGHLIGHTING
     _cmdSyntaxHighlight->setEnabled(enable);
 #endif
-    _cmdTileVert->setEnabled(enable);
-    _cmdTileHorz->setEnabled(enable);
-    _cmdTileFull->setEnabled(enable);
+    if (_mdiLayout) {
+        updateLayoutMode();
+    } else {
+        _cmdTileVert->setEnabled(enable);
+        _cmdTileHorz->setEnabled(enable);
+        _cmdTileFull->setEnabled(enable);
+    }
     _cmdSearch->setEnabled(enable);
     _cmdReplace->setEnabled(enable);
 }
@@ -698,6 +739,32 @@ void Editor::replaceDialog() {
     }
 }
 
+void Editor::updateLayoutMode() {
+    if (_mdiLayout) {
+        _cmdTileVert->setEnabled(_mdiLayout->Mode() != MdiLayout::LayoutMode::TileV);
+        _cmdTileHorz->setEnabled(_mdiLayout->Mode() != MdiLayout::LayoutMode::TileH);
+        _cmdTileFull->setEnabled(_mdiLayout->Mode() != MdiLayout::LayoutMode::Base);
+    }
+}
+
+void Editor::setTerminalHeightMode(TerminalMode mode) {
+    terminalMode = mode;
+}
+
+void Editor::updateTerminalHeight() {
+#ifdef HAS_TUIWIDGETS_0_2_2
+    if (terminalMode == TerminalMode::modeAuto) {
+        _mdiLayout->setMode(MdiLayout::LayoutMode::Base);
+        if (_file) {
+            int tmp = _file->visualLineCount() + 4;
+            if(tmp > terminal()->height()) {
+                terminal()->setInlineHeight(tmp);
+            }
+        }
+    }
+#endif
+}
+
 FileWindow *Editor::newFile(QString filename) {
     FileWindow *win = createFileWindow();
     if (filename.size()) {
@@ -890,7 +957,7 @@ void Editor::commandLineExecute(QString cmd) {
             });
         }
     } else if (cmd == "help") {
-        _commandLineWidget->setCmdEntryText("suspend shell");
+        _commandLineWidget->setCmdEntryText("suspend shell fullscreen nofullscreen incsize");
         showCommandLine();
     } else if (cmd == "suspend") {
         ::raise(SIGTSTP);
@@ -901,4 +968,13 @@ void Editor::commandLineExecute(QString cmd) {
         (void)!system(qgetenv("SHELL"));
         term->unpauseOperation();
     }
+#ifdef HAS_TUIWIDGETS_0_2_2
+    else if (cmd == "incsize") {
+        terminal()->setInlineHeight(terminal()->inlineHeight() + 1);
+    } else if (cmd == "fullscreen") {
+        terminal()->setInline(false);
+    } else if (cmd == "nofullscreen") {
+        terminal()->setInline(true);
+    }
+#endif
 }
